Package: readMzXmlData
Version: 2.8.4
Date: 2025-10-17
Title: Reads Mass Spectrometry Data in mzXML Format
Authors@R: person("Sebastian", "Gibb", role=c("aut", "cre"),
    email="mail@sebastiangibb.de")
Depends: R (>= 4.2.0)
Imports: base64enc, digest, XML
Description: Functions for reading mass spectrometry data in mzXML format.
License: GPL (>= 3)
URL: https://strimmerlab.github.io/software/maldiquant/
        https://codeberg.org/sgibb/readMzXmlData/
BugReports: https://codeberg.org/sgibb/readMzXmlData/issues/
LazyLoad: yes
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-10-17 06:48:33 UTC; sebastian
Author: Sebastian Gibb [aut, cre]
Maintainer: Sebastian Gibb <mail@sebastiangibb.de>
Repository: CRAN
Date/Publication: 2025-10-17 07:10:03 UTC
