% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{readMzXmlData-package}
\alias{readMzXmlData}
\alias{readMzXmlData-package}
\title{The readMzXmlData Package}
\description{
The package reads mass spectrometry data in mzXML format. \cr
}
\details{
Main functions:

\code{\link{readMzXmlFile}}: Reads mass spectrometry data in mzXML format.

\code{\link{readMzXmlDir}}: Reads recursively mass spectrometry data in mzXML
format in a specific directory.

\code{\link{mqReadMzXml}}: Reads mass spectrometry data into MALDIquant.
}
\references{
See website: \url{https://strimmerlab.github.io/software/maldiquant/}
}
\seealso{
\code{\link{readMzXmlDir}}, \code{\link{readMzXmlFile}}
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\keyword{IO}
