% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/attribute-functions.R
\name{.attributeTimeToDouble}
\alias{.attributeTimeToDouble}
\title{Converts XML attributes.}
\usage{
.attributeTimeToDouble(attributes, attributeName, required = FALSE)
}
\arguments{
\item{attributes}{\code{character}, XML attributes}

\item{attributeName}{\code{character}, name XML attributes of attribute to
convert}

\item{required}{\code{logical}, throw an error if an required attribute is
missing.}
}
\value{
\code{double}
}
\description{
This function converts a XML time attribute to \code{double}.
}
\seealso{
\code{\link[readMzXmlData]{.attributeToString}},
 \code{\link[readMzXmlData]{.attributeToDouble}}
}
\author{
Sebastian Gibb \email{mail@sebastiangibb.de}
}
\keyword{internal}
